/*

This function is called if the BadBlockTable does not exist.
It detects the BadBlocks of the flash 
and indicates a Bad Block by 0xBD and a Valid Block by 0xA1

Parameter to be passed: BlockNo
Returns: None

*/



#include "Extern_Declaration.h"

/**********************************************************************/


void DetectBadBlock(unsigned int block)
{

	short Store_BlockNo; 
	Store_BlockNo = block;
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	// Page Read Command:
	
	*pNFC_CMD =  PageReadCmd[0];
	
	/*  
	
	Set up the address cycles depending on Block number passed.
	Page0 to be read here is 0. Accordingly decode the address
	
	Column: A0-A11
	Page: 	A12-A17
	Block: 	A18-A28
	
	
	1st Address Cycle: A0-A7
	2nd Address Cycle: A8-A11
	3rd Address Cycle: A12-A19
	4th Address Cycle: A20-A27
	5th Address Cycle: A28
	
	*/
	
	short temp_Page = 0;	// We are reading Page 0
	short temp_Block = block;
	
	//Column
	*pNFC_ADDR = 0;
	*pNFC_ADDR = 0;
	
	//Page
	
	temp_Page = block & 3;		// Get bits A18 and A19 from Block address
	temp_Page = temp_Page << 6;	// Shift to append with Page Address
	
	// First 2 bits of Block and 6 bits of Page (which is 0) are sent
	*pNFC_ADDR = temp_Page;	
	
	// check if write buffer is empty or not:
	
	while(!(*pNFC_STAT & WB_EMPTY));	
	
	// Next 8 bits of Block address. i.e till 1023. 
			
	block = temp_Block;
	block = block & 0x3FC;	// Mask to get A20-A27	
	block = block >> 2;		// Because A18 and A19 are already sent out
	
	*pNFC_ADDR = block;
	
	
	// Block address greater than 1023
	
	block = temp_Block;
	block = block & 0xFC00;		// Mask to get A28-A30
	block = block >> 10;		// Because A18-A27 are already sent out
	
	*pNFC_ADDR = block;

		
	// Next command
	
	*pNFC_CMD =  PageReadCmd[1];
	

	//Wait till NBUSY_Flag sets here.
	
	while(!NBUSY_Flag); 
	NBUSY_Flag = 0;
		
	
	/* 
	
	Now Random Data Output here: Give 2 add cycle for 
	column:
	
	
	Column: A0-A7
	Column: A8-A11: 
	
	Spare Area Starting address: 2048
	
	This Random Data Output is to read 1st byte
	
	*/
	
	short Spare_1st_Byte;
				
	while(!(*pNFC_STAT & WB_EMPTY));
	
	*pNFC_CMD = RandomDataOutputCmd[0];
	
	*pNFC_ADDR = Flash_Page_Size;		
	
	*pNFC_ADDR = (Flash_Page_Size >> 8); 
			
	*pNFC_CMD = RandomDataOutputCmd[1];
		
	
	while(!(*pNFC_STAT & WB_EMPTY));
		
	//trigger a read request to nand flash.
	
	*pNFC_DATA_RD = 0;
	while(!(*pNFC_IRQSTAT & RD_RDY));
	Spare_1st_Byte = *pNFC_READ;
	*pNFC_IRQSTAT |= RD_RDY;
	ssync();
	
	/* 
	
	Now Random Data Output here: Give 2 add cycle for 
	column:
	
	
	Column: A0-A7
	Column: A8-A11: 
	
	Spare Area address: 2053
	
	This Random Data Output is to read 6th byte
	
	*/
	
	short Spare_6th_Byte;
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	*pNFC_CMD = RandomDataOutputCmd[0];
	
	*pNFC_ADDR = (Flash_Page_Size+5);	
	
	*pNFC_ADDR = ((Flash_Page_Size + 5 )>> 8);
	
	*pNFC_CMD = RandomDataOutputCmd[1];
	

	//trigger a read request to nand flash.

	while(!(*pNFC_STAT & WB_EMPTY));
	
	*pNFC_DATA_RD = 0;
	while(!(*pNFC_IRQSTAT & RD_RDY));
	Spare_6th_Byte = *pNFC_READ;
	*pNFC_IRQSTAT |= RD_RDY;
	ssync();
	
			 
	//Bad Block condition
	
	if((Spare_1st_Byte != 0xFF) &&  (Spare_6th_Byte != 0xFF))
	{
		BadBlockTable[i] = 0xBD;		//0xBD indicates Bad Block
	
	}
	
	else
	{
		BadBlockTable[i] = 0xA1;		//0xA1 indicates Good Block
		
	}
	
	
}

	
/**********************************************************************/

